# The Rust Programming Language

Book to get started with the Rust programming language.

Available online: <https://doc.rust-lang.org/book/>

Available offline: `rustup docs --book`

## About

Rust is a multi-paradigm, general-purpose programming language. Rust emphasizes performance, type safety, and concurrency. Rust enforces memory safety—that is, that all references point to valid memory—without requiring the use of a garbage collector or reference counting present in other memory-safe languages.

Rust keeps track of who can read and write to memory. It knows when the program is using memory and immediately frees the memory once it is no longer needed.

- [Website](https://www.rust-lang.org/)
- Local Documentation: `rustup doc`
- Rust version used: v1.74.0 (`rustc --version`)

## Summary

1. [Getting Started](./chapter_1_getting_started)
2. [Programming a Guessing Game](./chapter_2_guessing_game)
3. [Common Programming Concepts](./chapter_3_common_concepts)
4. [Understanding Ownership](./chapter_4_ownership)
5. [Using Structs to Structure Related Data](./chapter_5_structs)
6. [Enums and Pattern Matching](./chapter_6_enums)
7. [Managing Growing Projects with Packages, Crates, and Modules](./chapter_7_packages_crates_modules)
8. [Common Collections](./chapter_8_common_collections)
9. [Error Handling](./chapter_9_error_handling)
10. [Generic Types, Traits, and Lifetimes](./chapter_10_generics_traits_lifetimes)
11. [Testing](./chapter_11_testing)
12. [An I/O Project: Building a Command Line Program: `minigrep`](./chapter_12_minigrep)
13. [Functional Language Features: Iterators and Closures](./chapter_13_iterators_and_closures)
